// $Id: RETestControl.java,v 1.1.1.1 2000/12/18 16:37:55 sjakob Exp $
/*
 * RETestControl.java - The controller class which responds to events 
 * generated by a RETestView object. For use in the RETest plugin for jEdit.
 * Copyright (C) 2000 Steve Jakob
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.event.*;
import javax.swing.*;
import org.gjt.sp.jedit.*;
import gnu.regexp.*;
/**
*	The class is designed for use with jEdit for testing regular expressions
*	as per the gnu.regexp implementation included with jEdit. It responds to
*	events generated by the view object, RETestView, and performs the actual 
*	tests.
*	@author  Steve Jakob
*/
public class RETestControl implements ActionListener, ItemListener
{
	private RETestView myView;
	private StatusPane sp;
	private View jeditView = null;
	private static RE slashRE = null;
	/**
	 * Perl5 syntax with character classes enabled.
	 * Borrowed from org.gjt.sp.jedit.search.RESearchMatcher for compatability.
	 * @since RETest 1.0.1
	 */
	public static final RESyntax RE_SYNTAX_JEDIT = new RESyntax(
		RESyntax.RE_SYNTAX_PERL5).set(RESyntax.RE_CHAR_CLASSES);


	//Constructors
	public RETestControl() {
		super();
		initialize();
	}
	
	/**
	 * Establish the jEdit view for which this object was created. It is 
	 * required to allow the RE string to be inserted into the current buffer.
	 * @param vw The current jEdit view.
	 */
	public void setJeditView(View vw)
	{
		jeditView = vw;
	}

	//Implementors
	/**
	 * Perform any required initialization of the RETestControl object. 
	 * Create the view object.
	 */
	private void initialize() {
		myView = new RETestView(this);
		sp = myView.getStatusField();
		myView.pack();
	}
	
	/**
	 * Display the RETestView object.
	 */
	public void displayView() {
		myView.setVisible(true);
	}
	
	/**
	 * Respond to the selection of one of the buttons in the button panel.
	 * @param evt The ActionEvent generated by the button press.
	 */
	public void actionPerformed(ActionEvent evt) {
		String btnPressed = ((JButton)evt.getSource()).getActionCommand();
		if (btnPressed.equals("Test")) {
			try {
				RE testRE = new RE(myView.getREString(), 0, RE_SYNTAX_JEDIT);
				String testString = myView.getTestString();
				String replaceString = myView.getReplaceString();
				String selection = myView.getSelectedFunction();
				if (selection.equals("isMatch"))
					testIsMatch(testRE, testString);
				if (selection.equals("getMatch"))
					testGetMatch(testRE, testString);
				if (selection.equals("getAllMatches"))
					testGetAllMatches(testRE, testString);
				if (selection.equals("substitute"))
					testSubstitute(testRE, testString, replaceString);
				if (selection.equals("substituteAll"))
					testSubstituteAll(testRE, testString, replaceString);
				if (selection.equals("getMatchEnumeration"))
					testGetAllMatches(testRE, testString);
			} catch (gnu.regexp.REException ree) {
				sp.setMsg("ERROR: Bad regular expression.");
			}
		}
		if (btnPressed.equals("Close")) {
			myView.setVisible(false);
		}
		if (btnPressed.equals("Insert")) {
			if (slashRE == null) {
				try {
					slashRE = new RE("\\\\");
				} catch (REException e) {
					sp.setMsg("RETestContol: Error creating slashRE.");
					System.out.println(e);
				}
			} else {
			jeditView.getTextArea().setSelectedText(slashRE.substituteAll(
							myView.getREString(),"\\\\"));
			}
		}
		if (btnPressed.equals("Help")) {
			myView.toggleHelpPanel();
		}
	}
	
	/**
	 * Respond to the selection of one of the functions in the function list.
	 * @param evt The ItemEvent generated by the selection.
	 */
	public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
			String selection = myView.getSelectedFunction();
			if (selection.equals("substitute") ||
			selection.equals("substituteAll")) {
				myView.setReplaceFieldEnabled(true);
			}
			else {
				myView.setReplaceFieldEnabled(false);
			}
		}
	}
	
	/**
	 * Test the entered regular expression against the entered test string.
	 * Set the status message field to indicate success or failure.
	 */
	private void testIsMatch(RE testRE, String testString) {
		if (testRE.isMatch(testString)) {
			sp.setSuccessMsg("We have a match!!!");
		} else {
			sp.setFailureMsg("Sorry, try again.");
		}
	}
	
	/**
	 * Test the entered regular expression against the entered test string.
	 * If a match exists, copy the test string to the status message field,
	 * highlighting the match.
	 */
	private void testGetMatch(RE testRE, String testString) {
		REMatch rem = testRE.getMatch(testString);
		if (rem != null) {
			sp.setMsg(testString);
			sp.addHighlight(rem.getStartIndex(), rem.getEndIndex());
		} else {
			sp.setFailureMsg("Sorry, try again.");
		}
	}
	
	/**
	 * Test the entered regular expression against the entered test string.
	 * If a match exists, copy the test string to the status message field,
	 * highlighting all matches.
	 */
	private void testGetAllMatches(RE testRE, String testString) {
		REMatch[] rem = testRE.getAllMatches(testString);
		if (rem.length > 0) {
			sp.setMsg(testString);
			for (int i=0;i < rem.length;i++) {
				sp.addHighlight(rem[i].getStartIndex(), rem[i].getEndIndex());
			}
		} else {
			sp.setFailureMsg("Sorry, try again.");
		}
	}
	
	/**
	 * Replace the first occurrence of the RE in the test string with the replace
	 * string. Display the result in the status message field.
	 */
	private void testSubstitute(RE testRE, String testString, String replaceString) {
		sp.setMsg(testRE.substitute(testString, replaceString));
	}
	
	/**
	 * Replace all occurrences of the RE in the test string with the replace
	 * string. Display the result in the status message field.
	 */
	private void testSubstituteAll(RE testRE, String testString, String replaceString) {
		sp.setMsg(testRE.substituteAll(testString, replaceString));
	}
	
}
	/*
	 * Change Log:
	 * $Log: RETestControl.java,v $
	 * Revision 1.1.1.1  2000/12/18 16:37:55  sjakob
	 * Initial import.
	 * Bug fix release for Character Classes (eg. [[:alpha:]]) problem.
	 *
	 * Revision 1.3  2000/02/12 06:17:03  sjakob
	 * Moved redundant RE creation code from each testXXX method
	 * to actionPerformed.
	 *
	 * Revision 1.2  2000/02/12 05:48:37  sjakob
	 * Changed status field from JTextField to a specialized JTextPane,
	 * to allow for formatted messages using pre-defined styles.
	 * Implemented remaining tests.
	 *
	 * Revision 1.1  2000/02/04 20:31:09  sjakob
	 * Decoupled controller from RETestView class.
	 *
	 */


